#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

import sys
import os
import string
import shutil
import pwd
import subprocess
import threading
import time
import logging
from aptdaemon import client
import dbus
from gi import require_version
require_version('Gtk', '3.0')
from gi.repository import Gio, GLib, GObject, Gtk, Gdk

logging.basicConfig(level=logging.DEBUG)

# CARGA DE FICHEROS DE CONFIGURACIÓN
import configparser
CONF_FILE='/etc/cga-herramienta-informacion/peripherals.conf'
peripherals = configparser.ConfigParser()
peripherals.read(CONF_FILE)

GObject.threads_init()
Gdk.threads_init()

class MainWindow:
	def __init__(self):
		self.create_mainwindow_app()
		self.create_actdata_window()
		self.diag_backend = Diagnostico(self)
		self.mainwindow_show()
		
	def create_mainwindow_app(self):		
		self.mainwindow = Gtk.Window(Gtk.WindowType.TOPLEVEL)
		self.mainwindow.set_size_request(650, 500)
		self.mainwindow.set_border_width(10)
		self.mainwindow.set_position(Gtk.WindowPosition.CENTER)
		self.mainwindow.set_title("Herramienta de información")
		#self.mainwindow.set_wmclass("Herramienta de información", "Herramienta de información")
		self.mainwindow.set_icon_from_file('/usr/share/cga-herramienta-informacion/cga-herramienta-informacion.svg')

		#Senales de destroy y delete_event
		self.mainwindow.connect("delete_event", self.delete_event)
		self.mainwindow.connect("destroy", self.destroy)

		self.box = Gtk.VBox(False, 10)
		self.mainwindow.add(self.box)
		
		self.notebook = Gtk.Notebook()
		self.notebook.set_show_border(True)
		self.box.pack_start(self.notebook, True, True, 0)

		self.pagegeneral_scrolled = Gtk.ScrolledWindow()
		self.pagegeneral_scrolled.set_vexpand(True)
		self.pagegeneral = Gtk.TreeView()
		self.treestore_general = Gtk.TreeStore(str)
		self.pagegeneral.set_model(self.treestore_general)
		self.tvcolumn_general = Gtk.TreeViewColumn('')
		self.pagegeneral.append_column(self.tvcolumn_general)
		self.cell_general = Gtk.CellRendererText()
		self.tvcolumn_general.pack_start(self.cell_general, True)
		self.tvcolumn_general.add_attribute(self.cell_general, 'text', 0)
		self.pagegeneral_scrolled.add(self.pagegeneral)
		self.notebook.append_page(self.pagegeneral_scrolled, Gtk.Label('General'))
		
		self.pagered_scrolled = Gtk.ScrolledWindow()
		self.pagered_scrolled.set_vexpand(True)
		self.pagered = Gtk.TreeView()
		self.treestore_red = Gtk.TreeStore(str)
		self.pagered.set_model(self.treestore_red)
		self.tvcolumn_red = Gtk.TreeViewColumn('')
		self.pagered.append_column(self.tvcolumn_red)
		self.cell_red = Gtk.CellRendererText()
		self.tvcolumn_red.pack_start(self.cell_red, True)
		self.tvcolumn_red.add_attribute(self.cell_red, 'text', 0)
		self.pagered_scrolled.add(self.pagered)
		self.notebook.append_page(self.pagered_scrolled, Gtk.Label('Red'))

		self.pagehardware_scrolled = Gtk.ScrolledWindow()
		self.pagehardware_scrolled.set_vexpand(True)
		self.pagehardware = Gtk.TreeView()
		self.treestore_hardware = Gtk.TreeStore(str)
		self.pagehardware.set_model(self.treestore_hardware)
		self.tvcolumn_hardware = Gtk.TreeViewColumn('')
		self.pagehardware.append_column(self.tvcolumn_hardware)
		self.cell_hardware = Gtk.CellRendererText()
		self.tvcolumn_hardware.pack_start(self.cell_hardware, True)
		self.tvcolumn_hardware.add_attribute(self.cell_hardware, 'text', 0)
		self.pagehardware_scrolled.add(self.pagehardware)
		self.notebook.append_page(self.pagehardware_scrolled, Gtk.Label('Hardware'))

		self.pageperifericos_scrolled = Gtk.ScrolledWindow()
		self.pageperifericos_scrolled.set_vexpand(True)
		self.pageperifericos = Gtk.TreeView()
		self.treestore_perifericos = Gtk.TreeStore(str)
		self.pageperifericos.set_model(self.treestore_perifericos)
		self.tvcolumn_perifericos = Gtk.TreeViewColumn('')
		self.pageperifericos.append_column(self.tvcolumn_perifericos)
		self.cell_perifericos = Gtk.CellRendererText()
		self.tvcolumn_perifericos.pack_start(self.cell_perifericos, True)
		self.tvcolumn_perifericos.add_attribute(self.cell_perifericos, 'text', 0)
		self.pageperifericos_scrolled.add(self.pageperifericos)
		self.notebook.append_page(self.pageperifericos_scrolled, Gtk.Label('Periféricos'))

		self.pagesoftware = Gtk.VBox(False, 10)
		self.tvsoftware_scrolled = Gtk.ScrolledWindow()
		self.tvsoftware_scrolled.set_vexpand(True)
		self.tvsoftware = Gtk.TreeView()
		self.treestore_software = Gtk.TreeStore(str)
		self.tvsoftware.set_model(self.treestore_software)
		self.tvcolumn_software = Gtk.TreeViewColumn('')
		self.tvsoftware.append_column(self.tvcolumn_software)
		self.cell_software = Gtk.CellRendererText()
		self.tvcolumn_software.pack_start(self.cell_software, True)
		self.tvcolumn_software.add_attribute(self.cell_software, 'text', 0)
		self.tvsoftware_scrolled.add(self.tvsoftware)
		self.pagesoftware.pack_start(self.tvsoftware_scrolled, True, True, 0)
		self.buttonactsoft = Gtk.Button("Actualizar Software")
		self.buttonactsoft.connect("clicked", self.btnactualizar_clicked)
		self.pagesoftware.pack_start(self.buttonactsoft, False, False, 0)
		self.notebook.append_page(self.pagesoftware, Gtk.Label('Software'))

		self.buttonbox = Gtk.Box(spacing = 10)
		self.box.pack_start(self.buttonbox, False, True, 0)
		self.buttonsavefile = Gtk.Button("Guardar Archivo")
		self.buttonactdata = Gtk.Button("Actualizar Datos")
		self.buttonquit = Gtk.Button("Salir")
		self.buttonquit.connect("clicked", self.destroy)
		self.buttonactdata.connect("clicked", self.get_actdatainfo_callback)
		self.buttonsavefile.connect("clicked", self.save_file_info)
		self.buttonbox.pack_start(self.buttonsavefile,True, True, 0)
		self.buttonbox.pack_start(self.buttonactdata,True, True, 0)
		self.buttonbox.pack_start(self.buttonquit,True, True, 0)


	def create_actdata_window(self):
		self.actdatawindow = Gtk.Window(Gtk.WindowType.TOPLEVEL)
		self.actdatawindow.set_size_request(300, 200)
		self.actdatawindow.set_border_width(10)
		self.actdatawindow.set_position(Gtk.WindowPosition.CENTER)
		self.actdatawindow.set_title("Cargando Datos...")
		self.actdatawindow.set_transient_for(self.mainwindow)
		self.actdatawindow.set_modal(True)
		self.actdatawindow.set_decorated(False)
		self.actdatawindow.connect("destroy", self.actdatawindow_hide)
		self.actdatawindow.connect("delete_event", self.actdatawindow_hide)
		self.actvbox = Gtk.VBox(False, 10, spacing = 20)
		self.actdatawindow.add(self.actvbox)
		self.actlabel = Gtk.Label("Cargando datos de información")
		self.actvbox.pack_start(self.actlabel, True, True, 0)
	
		self.actvboxspinner = Gtk.VBox(False, 5)
		self.actspinner = Gtk.Spinner()
		self.actspinner.set_size_request(60, 60)
		self.actvboxspinner.pack_start(self.actspinner, True,False, 0)
		self.actspinner_complete_image = Gtk.Image()
		self.actspinner_complete_image.set_from_icon_name("process-completed-symbolic", 64)
		self.actvboxspinner.pack_start(self.actspinner_complete_image, True, False, 0)
		self.actvbox.pack_start(self.actvboxspinner, True, True, 0)
	
		self.actlabelvbox = Gtk.VBox(True, 10)	
		self.actlabelgeneral = Gtk.Label('<span foreground="red">Obteniendo información general</span>')
		self.actlabelgeneral.set_use_markup(True)
		self.actlabelred = Gtk.Label('<span foreground="red">Obtenidiendo datos de redes</span>')
		self.actlabelred.set_use_markup(True)
		self.actlabelhardware = Gtk.Label('<span foreground="red">Obteniendo datos de hardware</span>')
		self.actlabelhardware.set_use_markup(True)
		self.actlabelperifericos = Gtk.Label('<span foreground="red">Obteniendo datos de periféricos</span>')
		self.actlabelperifericos.set_use_markup(True)
		self.actlabelsoftware = Gtk.Label('<span foreground="red">Obteniendo datos de software</span>')
		self.actlabelsoftware.set_use_markup(True)
		self.actlabelvbox.pack_start(self.actlabelgeneral, False, True, 0)
		self.actlabelvbox.pack_start(self.actlabelred, False, True, 0)
		self.actlabelvbox.pack_start(self.actlabelhardware, False, True, 0)
		self.actlabelvbox.pack_start(self.actlabelperifericos, False, True, 0)
		self.actlabelvbox.pack_start(self.actlabelsoftware, False, True, 0)
		self.actvbox.pack_start(self.actlabelvbox, True, True, 0)
		self.finishbutton = Gtk.Button("Terminado")
		self.finishbutton.set_sensitive(False)
		self.actvbox.pack_start(self.finishbutton, False, True, 0)
		self.finishbutton.connect("clicked", self.actdatawindow_hide)
		
	def get_actdatainfo_callback(self, widget, data = None):
		self.create_actdata_window()
		self.actdatawindow_show()

	def get_actdatainfo(self):
		self.create_actdata_window()
		self.actdatawindow_show()

	def mainwindow_show(self):
		self.mainwindow.show_all()

	def actdatawindow_show(self):
		self.actdatawindow.show_all()
		thread_update = threading.Thread(target=self.diag_backend.update_data_info)
		thread_update.daemon = True
		thread_update.start()

	def actdatawindow_hide(self, widget, data = None):
		self.actdatawindow.destroy()
	
	def save_file_info(self, widget, data = None):
		filechooserdialog = Gtk.FileChooserDialog("Salvar fichero de datos", parent = self.mainwindow, action = Gtk.FileChooserAction.SAVE, buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
		filechooserdialog.set_current_folder(os.path.expanduser("~"))
		filtro = Gtk.FileFilter()
		filtro.set_name(".txt")
		filtro.add_pattern("*.txt")
		filechooserdialog.add_filter(filtro)
		response = filechooserdialog.run()
		if response == Gtk.ResponseType.OK:
			filename = filechooserdialog.get_filename()
			shutil.copy("/tmp/Información del Sistema.txt", filename)
			uid_usuario = pwd.getpwnam(sys.argv[1])[2]
			uid_grupo = pwd.getpwnam(sys.argv[1])[3]
			os.chown(filename, uid_usuario,uid_grupo)
			filechooserdialog.destroy()
		else:
			filechooserdialog.destroy()

	#Llama a la aplicación cga-actualizacion
	def btnactualizar_clicked(self, widget,data=None):
		self.buttonactsoft.set_sensitive(False)
		threading.Thread(target=self.eject_update).start()

	def eject_update(self):
		some_rlock = threading.RLock()
		with some_rlock:
			return_code = subprocess.call(["/usr/bin/cga-update-manager"])
			if return_code == 0:
				GObject.idle_add(self.buttonactsoft.set_sensitive, True)

	def delete_event(self, widget, data = None):
		Gtk.main_quit()

	def destroy(self, widget, data = None):
		Gtk.main_quit()


class Diagnostico:
	def __init__(self, frontend):
		self.appfrontend = frontend
		self.crear_fichero()

	def update_data_info(self):
		GObject.idle_add(self.appfrontend.treestore_general.clear)
		GObject.idle_add(self.appfrontend.treestore_red.clear)
		GObject.idle_add(self.appfrontend.treestore_hardware.clear)
		GObject.idle_add(self.appfrontend.treestore_perifericos.clear)
		GObject.idle_add(self.appfrontend.treestore_software.clear)
		GObject.idle_add(self.appfrontend.actspinner_complete_image.hide)
		GObject.idle_add(self.appfrontend.actspinner.start)
		self.general()
		self.red()
		self.hardware()
		self.perifericos()
		self.software()
		GObject.idle_add(self.appfrontend.actspinner.stop)
		GObject.idle_add(self.appfrontend.actspinner.hide)
		GObject.idle_add(self.appfrontend.actspinner_complete_image.show)
		GObject.idle_add(self.appfrontend.finishbutton.set_sensitive, True)

	#Crea el archivo vacío donde se guardará la información
	def crear_fichero(self):
		f = open("/tmp/Información del Sistema.txt","w")
		f.close()

	#Función a la q le pasamos un comando y nos devuelve el resultado en una lista con lo que saldría por terminal
	def leer_salida(self, comando):
		p = subprocess.Popen(comando, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
		(stdin, var) = (p.stdin, p.stdout)
		datos=[]
		for line in var.readlines():
			datos.append(line.decode("utf-8"))
		return datos

	#Procedimiento que va añadiendo información al archivo
	def informe(self, datos):
		f = open("/tmp/Información del Sistema.txt","a")
		f.write(datos+"\n")
		f.close()

	def general(self):
		########### GENERAL ############
		self.informe("*** INFORMACION GENERAL ***")

		#Escuela 2.0
		escuela = ""
		comando="dpkg -l |grep -i guadalinexedu-educacion-aula20 |cut -d' ' -f3"
		milistado=self.leer_salida(comando)
		if milistado:
			escuela = "Escuela 2.0"
		#Distribución
		comando="lsb_release -cs"
		milistado=self.leer_salida(comando)
		for linea in milistado:
			if str(linea[:-1]) == "lucid":#"2.6.32-23-generic":
				comando="dpkg -l|grep guadalinexedu-dom-app-install-data"
				milistado=self.leer_salida(comando)
				if milistado:
					piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu 10.04 versión doméstica'])
					self.informe("-Distribución: Guadalinex Edu 10.04 versión doméstica")
				else:
					piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu 10.04 Centros Educativos '+ escuela])
					self.informe("-Distribución: Guadalinex Edu 10.04 Centros Educativos "+ escuela)
			elif str(linea[:-1]) == "precise":#"3.2.0-36-generic-pae":
				comando="dpkg -l|grep guadalinexedu-dom-app-install-data"
				milistado=self.leer_salida(comando)
				if milistado:
					piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu 2013 versión doméstica'])
					self.informe("-Distribución: Guadalinex Edu 2013 versión doméstica")
				else:
					piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu 2013 Centros Educativos '+ escuela])
					self.informe("-Distribución: Guadalinex Edu 2013 Centros Educativos "+ escuela)
			elif str(linea[:-1]) == "jaunty":#"2.6.28-16-generic":
				comando="dpkg -l|grep guadalinex-meta"
				milistado=self.leer_salida(comando)
				if milistado:
					piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu 10.04 versión doméstica'])
					self.informe("-Distribución: Guadalinex Edu 9.04 versión doméstica")
				else:
					piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu 9.04 Centros Educativos '+ escuela])
					self.informe("-Distribución: Guadalinex Edu 9.04 Centros Educativos "+ escuela)
			elif str(linea[:-1]) == "xenial":
				comando="dpkg -l|grep guadalinex-meta"
				milistado=self.leer_salida(comando)
				if milistado:
					comando="dpkg -l|grep guadalinexedu-educacion-slim"
					milistado=self.leer_salida(comando)
					if milistado:
						piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu Slim versión doméstica'])
						self.informe("-Distribución: Guadalinex Edu Slim versión doméstica")
					else:
						piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu Next versión doméstica'])
						self.informe("-Distribución: Guadalinex Edu Next versión doméstica")
				else:
					comando="dpkg -l|grep guadalinexedu-educacion-tic-slim"
					milistado=self.leer_salida(comando)
					if milistado:
						piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu Slim Centros Educativos '+ escuela])
						self.informe("-Distribución: Guadalinex Edu Slim Centros Educativos "+ escuela)
					else:
						piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: Guadalinex Edu Next Centros Educativos '+ escuela])
						self.informe("-Distribución: Guadalinex Edu Next Centros Educativos "+ escuela)
			else:
				piter_general = self.appfrontend.treestore_general.append(None, ['Distribución: desconocida'])
				self.informe("-Distribución: desconocida")

			break

		#Usuario
		piter_general = self.appfrontend.treestore_general.append(None, ['Usuario autenticado: '+sys.argv[1]])
		self.informe("-Usuario autenticado: "+sys.argv[1])
		
		#Dual
		comando = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","dual"])
		milistado=self.leer_salida(comando)
		if milistado:
			piter_general = self.appfrontend.treestore_general.append(None, ['El equipo es dual'])
			self.informe("-El equipo es dual")

		GObject.idle_add(self.appfrontend.actlabelgeneral.set_markup, '<span foreground="green">Obteniendo información general</span>')
	
	def red(self):
		########### RED ############
		self.informe("\n*** RED ***")
		#Contar cuantas redes hay conectadas
		comando1="ifconfig |grep -c Ethernet |cut -d' ' -f1"
		milistado1=self.leer_salida(comando1)
		#Nombre de red(eth0,wlan0,...)
		comando2="ifconfig |grep Ethernet |cut -d' ' -f1"
		milistado2=self.leer_salida(comando2)
		#Mientras milistado1 de redes conectadas sea mayor que 0
		if int(milistado1[0]) > 0:
			contador = int(milistado1[0])
			#Mientras contador sea mayor 0
			while contador > 0:
				contador = contador -1

				piter_red = self.appfrontend.treestore_red.append(None, ['Interfaz de red: '+str(milistado2[contador].rstrip())])
				self.informe("* "+str(milistado2[contador].rstrip())+" *")
				#IP
				comando="ifconfig "+str(milistado2[contador].rstrip())+" |grep -i Direc.\ \inet|cut -d':' -f2|cut -d 'D' -f1"
				milistado=self.leer_salida(comando)
				if milistado:
					for linea in milistado:
						piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Dirección IP: '+linea[:-1]])
						self.informe("-Dirección IP: "+linea[:-1])
				else:
					piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Dirección IP: No tiene ip asignada'])
					self.informe("-Dirección IP: No tiene ip asignada")

				#Máscara
				comando="ifconfig "+str(milistado2[contador].rstrip())+" |grep -i Másc|cut -d':' -f4"
				milistado=self.leer_salida(comando)
				if milistado:
					for linea in milistado:
						piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Máscara de red: '+linea[:-1]])
						self.informe("-Máscara de red: "+linea[:-1])
				else:
					piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Máscara de red: No tiene máscara asignada'])
					self.informe("-Máscara de red: No tiene máscara asignada")

				#MAC
				comando="ifconfig "+str(milistado2[contador].rstrip())+"|grep -i direcciónHW|cut -d'W' -f2"
				milistado=self.leer_salida(comando)
				for linea in milistado:
					piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Dirección MAC: '+linea[:-1]])
					self.informe("-Dirección MAC: "+linea[:-1])

				#Comprueba si la red esta conectada por cable o wifi
				comando="cat /proc/net/wireless |grep "+str(milistado2[contador].rstrip())
				milistado=self.leer_salida(comando)
				if milistado:
					piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Conexión habilitada por wifi'])
					self.informe("-Conexión habilitada por wifi")
					#Si es por wifi busca el ESSID de la red
					comando="iwconfig "+str(milistado2[contador].rstrip())+" |grep ESSID:| cut -d':' -f2"
					milistado=self.leer_salida(comando)
					for linea in milistado:
						if str(linea[1:-4].rstrip()):
							piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['ESSID: '+linea[1:-4]])
							self.informe("-ESSID: "+linea[1:-4])
						else:
							piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['ESSID: No esta conectado a ninguna red wifi'])
							self.informe("-ESSID: No esta conectado a ninguna red wifi")
				else:
					piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Conexión habilitada por cable'])
					self.informe("-Conexión habilitada por cable")

				#Subred a la que pertenece
				comando="ifconfig "+str(milistado2[contador].rstrip())+"|grep -i Direc.\ \inet|cut -d'.' -f4"
				milistado=self.leer_salida(comando)
				if milistado:
					for linea in milistado:
						if int(linea[:-1]) >=8 and int(linea[:-1]) <=15:
							piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Subred: Alumno'])
							self.informe("-Subred: Alumno")
						elif int(linea[:-1]) >=4 and int(linea[:-1]) <=7:
							piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Subred: Gestión'])
							self.informe("-Subred: Gestión")
						elif int(linea[:-1]) >=0 and int(linea[:-1]) <=3:
							piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Subred: Profesor'])
							self.informe("-Subred: Profesor")
						else:
							piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Subred: desconocida'])
							self.informe("-Subred: desconocida")
							break
				else:
					piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Subred: Al no tener ip asignada no se sabe a que subred pertenece'])
					self.informe("-Subred: Al no tener ip asignada no se sabe a que subred pertenece")

				#Si posee infraestructuras TIC
				piter_red_hijo = self.appfrontend.treestore_red.append(piter_red, ['Infraestructura TIC'])
				comando="ping -c 1 f0 |grep -i received |cut -d',' -f2"
				milistado=self.leer_salida(comando)
				if str(milistado[0].rstrip()) == " 1 received":
					self.informe("-Infraestructura TIC: ")
					self.appfrontend.treestore_red.append(piter_red_hijo, ['f0'])
					self.informe("f0")
					comando="ping -c 1 c0 |grep -i received |cut -d',' -f2"
					milistado=self.leer_salida(comando)
					if str(milistado[0].rstrip()) == " 1 received":
						self.appfrontend.treestore_red.append(piter_red_hijo, ['c0'])
						self.informe("c0")
				else:
					self.appfrontend.treestore_red.append(piter_red_hijo, ['No se ha encontrado ninguna infraestructura TIC'])
					self.informe("-Infraestructura TIC: No se ha encontrado ninguna infraestructura TIC")

		GObject.idle_add(self.appfrontend.actlabelred.set_markup, '<span foreground="green">Obteniendo información de redes</span>')

	def hardware(self):
		########### HARDWARE ###########
		self.informe("\n*** HARDWARE ***")
		piter_hardware = self.appfrontend.treestore_hardware.append(None, ['Placa base'])
		self.informe("-Placa Base: ")
		#Nombre, serial, fabricante y versión de la placa base
		salida = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","motherboard"])
		milistado=[salida.decode("utf-8")]
		for linea in milistado:
			if str(linea[:-1].rstrip()):
				self.appfrontend.treestore_hardware.append(piter_hardware, [linea[:-1]])
				self.informe(linea[:-1])

		piter_hardware = self.appfrontend.treestore_hardware.append(None, ['Tarjeta gráfica'])
		self.informe("-Tarjeta gráfica: ")
		#Tarjetas gráficas
		comando="lspci |grep -i VGA\ \compatible\ \controller:|cut -d':' -f3"
		milistado=self.leer_salida(comando)
		for linea in milistado:
			if str(linea[:-1].rstrip()):
				self.appfrontend.treestore_hardware.append(piter_hardware, [linea[:-1]])
				self.informe(linea[:-1])

		piter_hardware = self.appfrontend.treestore_hardware.append(None, ['Tarjeta de red'])
		self.informe("-Tarjeta de red: ")
		#Tarjetas de red
		comando="lspci |grep -i Ethernet\ \controller:|cut -d':' -f3 && lspci|grep -i Network\ \controller:|cut -d':' -f3"
		milistado=self.leer_salida(comando)
		for linea in milistado:
			if str(linea[:-1].rstrip()):
				self.appfrontend.treestore_hardware.append(piter_hardware, [linea[:-1]])
				self.informe(linea[:-1])

		piter_hardware = self.appfrontend.treestore_hardware.append(None, ['Tarjeta de sonido'])
		self.informe("-Tarjeta de sonido: ")
		#Tarjetas de sonido
		comando="lspci |grep -i Audio\ \device:|cut -d':' -f3"
		milistado=self.leer_salida(comando)
		for linea in milistado:
			if str(linea[:-1].rstrip()):
				self.appfrontend.treestore_hardware.append(piter_hardware, [linea[:-1]])
				self.informe(linea[:-1])

		piter_hardware = self.appfrontend.treestore_hardware.append(None, ['Disco duro'])
		self.informe("-Disco duro: ")
		#Disco duro
		salida = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","harddisk"])
		milistado=[salida.decode("utf-8")]
		for linea in milistado:
			if str(linea[:-1].rstrip()):
				self.appfrontend.treestore_hardware.append(piter_hardware, [linea[:-1]])
				self.informe(linea[:-1])

		GObject.idle_add(self.appfrontend.actlabelhardware.set_markup, '<span foreground="green">Obteniendo información de hardware</span>')

	def perifericos(self):
		########### PERIFERICOS ############
		self.informe("\n*** PERIFERICOS ***")
		
		for manufacturer in peripherals.sections():
			############ PIZARRAS DIGITALES ############
			stringpdi = peripherals.get(manufacturer, 'string_name')
			piter_perifericos = self.appfrontend.treestore_perifericos.append(None, [stringpdi])
			self.informe("-{0}:".format(stringpdi))
			#Busca si esta instalado el meta de la pizarra en cuestión
			comando="dpkg -l |grep -i {0}|cut -d' ' -f3".format(peripherals.get(manufacturer, 'meta'))
			milistado=self.leer_salida(comando)
			if milistado:
				self.appfrontend.treestore_perifericos.append(piter_perifericos, ['Se ha detectado software'])
				self.informe("Se ha detectado software")
			else:
				self.appfrontend.treestore_perifericos.append(piter_perifericos, ['No se detecta software'])
				self.informe("No se detecta software")

			#Busca si se encuentra conectada al equipo
			modelos=eval(peripherals.get(manufacturer, 'products'))
			try:
				encontrada=False
				for pdi in modelos:
					comando="lsusb |grep {0}".format(pdi['id'])
					milistado=self.leer_salida(comando)
					if milistado:
						mensaje='Está conectada. Modelo: {0}'.format(pdi['name'])
						self.appfrontend.treestore_perifericos.append(piter_perifericos, [mensaje])
						self.informe(mensaje)
						encontrada=True
				if not encontrada:
					raise Exception('No está conectada')

			except Exception as e:
				self.appfrontend.treestore_perifericos.append(piter_perifericos, [format(e)])
				self.informe(format(e))

		############ IMPRESORAS ############
		piter_perifericos = self.appfrontend.treestore_perifericos.append(None, ['Impresoras'])
		self.informe("-Impresoras: ")
		#Si existe el fichero printers.conf
		if os.path.exists("/etc/cups/printers.conf"):
			#Busca el número total de impresoras
			salida = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","printer1"])
			milistado=[salida.decode("utf-8")]
			contador_impr = int(milistado[0])
			#Busca todas las impresoras configuradas en el equipo
			salida = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","printer2"])
			milistado=[salida.decode("utf-8")]
			if milistado:
				for linea in milistado:
					if str(linea[:-1].rstrip()) != "PDF":
						self.appfrontend.treestore_perifericos.append(piter_perifericos, [linea[:-1]])
						self.informe(linea[:-1])
					elif str(linea[:-1].rstrip()) == "PDF" and contador_impr == 1:
						piter_perifericos = self.appfrontend.treestore_perifericos.append(piter_perifericos, ['No hay impresoras configuradas'])
						self.informe("No hay impresoras configuradas")
					else:
						piter_perifericos = self.appfrontend.treestore_perifericos.append(piter_perifericos, ['No hay impresoras configuradas'])
						self.informe("No hay impresoras configuradas")
		else:
			piter_perifericos = self.appfrontend.treestore_perifericos.append(piter_perifericos, ['No hay impresoras configuradas'])
			self.informe("No hay impresoras configuradas")

		############ DISPOSITIVOS USB ############
		piter_perifericos = self.appfrontend.treestore_perifericos.append(None, ['Dispositivos USB'])
		self.informe("-Dispositivos USB: ")
		#Busca los dispositivos usb conectados al equipo
		comando="lsusb |grep -v  Linux|cut -d' ' -f6"
		milistado=self.leer_salida(comando)
		if milistado:
			for linea_id in milistado:
				#Busca el nombre del dispositivo
				comando="lsusb |grep "+str(linea_id[:-1].rstrip())+"|cut -d' ' -f7-20"
				milistado=self.leer_salida(comando)
				for linea_nombre in milistado:
					if str(linea_nombre[:-1].rstrip()):
						#Busco el tipo de dispositivo que es
						salida = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","usb", linea_id[:-1]])
						milistado=[salida.decode("utf-8")]
						for linea in milistado:
							#Esta cadena aparece en los pendrives y los hd externos, si no encuentra la cadena, entra en el if
							if int(linea[:-1].find("Connection timed out")) == -1:
								self.appfrontend.treestore_perifericos.append(piter_perifericos, [linea_nombre[:-1]+", "+linea[:-1]])
								self.informe(linea_nombre[:-1]+", "+linea[:-1])
					else:
						#Busco el tipo de dispositivo que es
						salida = subprocess.check_output(["sudo","/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-hardware.sh","usb", linea_id[:-1]])
						milistado=[salida.decode("utf-8")]
						for linea in milistado:
							#Esta cadena aparece en los pendrives y los hd externos, si no encuentra la cadena, entra en el if
							if int(linea[:-1].find("Connection timed out")) == -1:
								self.appfrontend.treestore_perifericos.append(piter_perifericos, ['Dispositivo desconocido, '+linea[:-1]])
								self.informe('Dispositivo desconocido, '+linea[:-1])
		else:
			self.appfrontend.treestore_perifericos.append(piter_perifericos, ['No hay dispositivos USB conectados al equipo'])
			self.informe("No hay dispositivos USB conectados al equipo")

		GObject.idle_add(self.appfrontend.actlabelperifericos.set_markup, '<span foreground="green">Obteniendo información de periféricos</span>')

	def software(self):
		########### SOFTWARE ############
		self.informe("\n*** SOFTWARE ***")
		correcto = True
		#Busca los paquetes que no se han podido borrar archivos de configuración
		comando="dpkg -l |grep ^.f |cut -d ' ' -f3"
		milistado=self.leer_salida(comando)
		if milistado:
			Correcto = False
			piter_software = self.appfrontend.treestore_software.append(None, ['Paquetes que no se han podido borrar archivos de configuración'])
			self.informe("-Paquetes que no se han podido borrar archivos de configuración: ")
			for linea in milistado:
				self.appfrontend.treestore_software.append(piter_software, [linea[:-1]])
				self.informe(linea[:-1])

		#Busca los paquetes que están parcialmente instalados
		comando="dpkg -l |grep ^.h |cut -d ' ' -f3"
		milistado=self.leer_salida(comando)
		if milistado:
			Correcto = False
			piter_software = self.appfrontend.treestore_software.append(None, ['Paquetes parcialmente instalados'])
			self.informe("-Paquetes parcialmente instalados: ")
			for linea in milistado:
				self.appfrontend.treestore_software.append(piter_software, [linea[:-1]])
				self.informe(linea[:-1])

		#Si no encuentra nada
		if correcto == True:
			piter_software = self.appfrontend.treestore_software.append(None, ['No se presentan inconsistencias en el software del equipo'])
			self.informe("No se presentan inconsistencias en el software del equipo")

		GObject.idle_add(self.appfrontend.actlabelsoftware.set_markup, '<span foreground="green">Obteniendo información de software</span>')

	#Método para verificar si ya hay una transacción apt corriendo
	def get_transaction(self):
		bus = dbus.SystemBus()
		apt_daemon = client.get_aptdaemon(bus=bus)
		current, queued = apt_daemon.GetActiveTransactions()
		if current:
			self.btnactualizar.set_sensitive(False)
		else:
			self.btnactualizar.set_sensitive(True)

		return True

	def get_update_manager(self):
		#Se ejecuta el comando ps ax|grep cga-update-manager|wc -l
		return_code = subprocess.check_output(["bash", "/usr/share/cga-herramienta-informacion/cga-herramienta-informacion-update.sh"])
		if int(return_code.strip()) < 2:
			self.btnactualizar.set_sensitive(True)
		else:
			self.btnactualizar.set_sensitive(False)
		return True

def main(widget=None, data=None):
	win = MainWindow()
	win.get_actdatainfo()

if __name__ == "__main__":
	main()
	Gtk.main()
